@@ -149,11 +149,15 @@ impl<A: MemPool> BuddyAlg<A> {
         self.mutex = 0; }
     }
 
+    #[inline]
+    fn in_range<'a>(off: u64) -> bool {
+        (off < u64::MAX - A::start()) && (off + A::start() < A::end())
+    }
+
     #[inline]
     #[track_caller]
     fn buddy<'a>(off: u64) -> &'a mut Buddy {
-        debug_assert!(off < u64::MAX - A::start(), "off(0x{:x}) out of range", off);
-        debug_assert!(off + A::start() < A::end(), "off(0x{:x}) out of range", off);
+        debug_assert!(Self::in_range(off), "off(0x{:x}) out of range", off);
         unsafe { read_addr(A::start() + off) }
     }
 
@@ -430,7 +434,7 @@ impl<A: MemPool> BuddyAlg<A> {
         self.lock();
 
         if !self.aux.is_empty() {
-            self.discard();
+            // self.discard();
             return true;
         }
 
@@ -487,6 +491,12 @@ impl<A: MemPool> BuddyAlg<A> {
         #[cfg(any(feature = "no_pthread", windows))] {
         self.mutex = 0; }
 
+
+        #[cfg(feature = "check_allocator_cyclic_links")]
+        if !self.verify() {
+            eprintln!("not verified before recovery");
+        }
+
         if self.aux_valid {
             #[cfg(debug_assertions)]
             eprintln!("Crashed while the allocator was operating");
@@ -532,6 +542,11 @@ impl<A: MemPool> BuddyAlg<A> {
             self.log64.clear();
             self.drop_log.clear();
         }
+
+        #[cfg(feature = "check_allocator_cyclic_links")]
+        if !self.verify() {
+            eprintln!("not verified after recovery");
+        }
     }
 
     pub fn recovery_info(&self, info_level: u32) -> String {
@@ -594,7 +609,7 @@ impl<A: MemPool> BuddyAlg<A> {
         self.foot_print
     }
 
-    fn check(&self, f: &str) {
+    pub fn check(&self, f: &str) {
         for idx in 3..self.last_idx + 1 {
             let mut curr = self.buddies[idx];
             while let Some(b) = off_to_option(curr) {
@@ -605,6 +620,32 @@ impl<A: MemPool> BuddyAlg<A> {
         }
     }
 
+    pub fn verify(&mut self) -> bool {
+        if std::env::var("VERIFY").is_err() { return true; }
+        let loops = std::env::var("VERIFY").unwrap() == "2";
+        self.lock();
+        for idx in 3..self.last_idx + 1 {
+            let mut curr = self.buddies[idx];
+            let mut links = vec![];
+            while let Some(b) = off_to_option(curr) {
+                if { if loops { links.contains(&b) } else { false } } || !Self::in_range(b) {
+                    self.unlock();
+                    if !Self::in_range(b) {
+                        eprintln!("Verification Failed: Invalid block address 0x{:x} (idx={})", b, idx);
+                    } else {
+                        eprintln!("Verification Failed: A cyclic link detected in list {}", idx);
+                    }
+                    return false;
+                }
+                if loops { links.push(b); }
+                let e = Self::buddy(b);
+                curr = e.next;
+            }
+        }
+        self.unlock(); 
+        true
+    }
+
     /// Prints the free lists
     pub fn print(&self) {
         println!();
@@ -716,8 +757,8 @@ mod test {
         struct Root {
             vec: PRefCell<PVec<Parc<(i32, PMutex<PString>)>>>
         }
-        impl Default for Root {
-            fn default() -> Self {
+        impl RootObj<P> for Root {
+            fn init(_: &Journal) -> Self {
                 Root {
                     vec: PRefCell::new(PVec::new())
                 }
@@ -994,7 +1035,7 @@ macro_rules! pool {
                 /// [`BuddyAlloc`](#) if success. The pool remains open as long
                 /// as the instance lives.
                 #[track_caller]
-                pub fn open_impl(filename: &str) -> Result<Self> {
+                pub fn open_impl(filename: &str, no_check: bool) -> Result<Self> {
                     let metadata = std::fs::metadata(filename);
                     if let Err(e) = &metadata {
                         Err(format!("{}", e))
@@ -1025,10 +1066,12 @@ macro_rules! pool {
                             let inner = unsafe {
                                 read::<BuddyAllocInner>(raw_offset)
                             };
-                            assert_eq!(
-                                inner.magic_number, id,
-                                "Invalid magic number for the pool image file"
-                            );
+                            if !no_check {
+                                assert_eq!(
+                                    inner.magic_number, id,
+                                    "Invalid magic number for the pool image file"
+                                );
+                            }
 
                             let base = raw_offset as *mut _ as u64;
                             unsafe {
@@ -1270,6 +1313,20 @@ macro_rules! pool {
                     })
                 }
 
+                #[inline]
+                #[allow(unused_unsafe)]
+                #[track_caller]
+                fn verify() -> bool {
+                    static_inner!(BUDDY_INNER, inner, {
+                        for i in 0..inner.zone.count() {
+                            if !inner.zone[i].verify() {
+                                return false;
+                            }
+                        }
+                        true
+                    })
+                }
+
                 #[inline]
                 #[allow(unused_unsafe)]
                 #[track_caller]
@@ -1344,14 +1401,28 @@ macro_rules! pool {
                         for i in 0..inner.zone.count() {
                             inner.zone[i].recover();
                         }
+
+                        #[cfg(feature = "check_allocator_cyclic_links")]
+                        debug_assert!(Self::verify());
+
                         while let Ok(logs) = Self::deref_mut::<Journal>(inner.journals) {
 
                             #[cfg(feature = "verbose")]
                             println!("{:?}", logs);
 
+                            #[cfg(feature = "check_allocator_cyclic_links")]
+                            debug_assert!(Self::verify());
+
                             logs.recover();
+
+                            #[cfg(feature = "check_allocator_cyclic_links")]
+                            debug_assert!(Self::verify());
+
                             logs.clear();
 
+                            #[cfg(feature = "check_allocator_cyclic_links")]
+                            debug_assert!(Self::verify());
+
                             #[cfg(feature = "pin_journals")]
                             Self::drop_journal(logs);
                         }
@@ -1413,8 +1484,10 @@ macro_rules! pool {
                     unsafe {
                         while OPEN.compare_exchange(false, true, Ordering::AcqRel, Ordering::Relaxed).is_err() {}
                         if !Self::running_transaction() {
-                            if let Ok(_) = Self::apply_flags(path, flags) {
-                                let res = Self::open_impl(path);
+                            if flags == open_flags::O_READINFO {
+                                Self::open_impl(path, true)
+                            } else if let Ok(_) = Self::apply_flags(path, flags) {
+                                let res = Self::open_impl(path, false);
                                 if res.is_ok() {
                                     Self::recover();
                                 }
@@ -1425,7 +1498,7 @@ macro_rules! pool {
                             }
                         } else {
                             OPEN.store(false, Ordering::Release);
-                            Err("Could not open a pool inside a transaction of its own kind"
+                            Err("An uncommitted transaction exists in the pool"
                                 .to_string())
                         }
                     }
