@@ -81,6 +81,9 @@ pub mod open_flags {
 
     /// Open Flag: Creates a pool memory file of size 64TB
     pub const O_64TB: u32 = 0x00100000;
+
+    /// Open Flag: Open only to read info
+    pub const O_READINFO: u32 = u32::MAX;
 }
 
 pub use open_flags::*;
@@ -379,6 +382,12 @@ where
         true
     }
 
+    /// Indicates if there the pool is in a good shape
+    #[inline]
+    fn verify() -> bool {
+        true
+    }
+
     /// Translates raw pointers to memory offsets
     ///
     /// # Safety
@@ -803,6 +812,41 @@ where
         p
     }
 
+    /// Allocates new memory and then copies `x` into it with `DropOnFailure` log
+    unsafe fn new_copy<'a, T: 'a>(x: &T, j: &Journal<Self>) -> &'a mut T 
+    where T: ?Sized {
+        let s = mem::size_of_val(x);
+        debug_assert!(s != 0, "Cannot allocated ZST");
+
+        let mut log = Log::drop_on_failure(u64::MAX, 1, j);
+        let (p, off, len, z) = Self::pre_alloc(s);
+        if p.is_null() {
+            panic!("Memory exhausted");
+        }
+        Self::drop_on_failure(off, len, z);
+        std::ptr::copy_nonoverlapping(x as *const T as *const u8, p, s);
+        log.set(off, len, z);
+        Self::perform(z);
+        utils::read(p)
+    }
+
+    /// Allocates new memory and then copies `x` into it with `DropOnFailure` log
+    unsafe fn new_copy_slice<'a, T: 'a>(x: &[T], j: &Journal<Self>) -> &'a mut [T] {
+        let s = mem::size_of_val(x);
+        debug_assert!(s != 0, "Cannot allocated ZST");
+
+        let mut log = Log::drop_on_failure(u64::MAX, 1, j);
+        let (p, off, len, z) = Self::pre_alloc(s);
+        if p.is_null() {
+            panic!("Memory exhausted");
+        }
+        Self::drop_on_failure(off, len, z);
+        std::ptr::copy_nonoverlapping(x as *const [T] as *const u8, p, s);
+        log.set(off, len, z);
+        Self::perform(z);
+        utils::read(p)
+    }
+
     /// Allocates new memory and then places `x` into it without realizing the allocation
     unsafe fn atomic_new<'a, T: 'a>(x: T) -> (&'a mut T, u64, usize, usize) {
         log!(Self, White, "ALLOC", "TYPE: {}", std::any::type_name::<T>());
@@ -1106,6 +1150,9 @@ where
         #[cfg(feature = "stat_perf")]
         let _perf = crate::stat::Measure::<Self>::Transaction;
         
+        #[cfg(feature = "check_allocator_cyclic_links")]
+        debug_assert!(Self::verify());
+
         let mut chaperoned = false;
         let cptr = &mut chaperoned as *mut bool;
         let res = std::panic::catch_unwind(|| {
@@ -1150,6 +1197,9 @@ where
         #[cfg(feature = "stat_perf")]
         let _perf = crate::stat::Measure::<Self>::Logging(std::time::Instant::now());
 
+        #[cfg(feature = "check_allocator_cyclic_links")]
+        debug_assert!(Self::verify());
+
         unsafe {
             crate::ll::sfence();
 
