@@ -124,7 +124,10 @@ impl<A: MemPool> Page<A> {
 
     unsafe fn rollback(&mut self) {
         for i in 0..self.len {
-            self.logs[i].rollback();
+            self.logs[self.len - i - 1].rollback();
+        }
+        for i in 0..self.len {
+            self.logs[i].rollback_drop_on_abort();
         }
     }
 
@@ -221,9 +224,14 @@ impl<A: MemPool> Journal<A> {
         self.next_off = *head_off;
         A::log64(A::off_unchecked(head_off), me, zone);
 
-        if let Ok(j) = A::deref_mut::<Journal<A>>(*head_off) {
+        if *head_off != u64::MAX {
+            let j = utils::read_addr::<Journal<A>>(A::start() + *head_off);
             A::log64(A::off_unchecked(&j.prev_off), me, zone);
+
         }
+        // if let Ok(j) = A::deref_mut::<Journal<A>>(*head_off) {
+        //     A::log64(A::off_unchecked(&j.prev_off), me, zone);
+        // }
     }
 
     #[inline]
@@ -423,9 +431,9 @@ impl<A: MemPool> Journal<A> {
             curr = page.next;
         }
         let mut curr = self.pages;
-        let fast_forward = self.fast_forward();
-        if !self.is_set(JOURNAL_COMMITTED) || fast_forward {
-            let rollback = !fast_forward || !self.is_set(JOURNAL_COMMITTED);
+        let resume = self.resume();
+        if !self.is_set(JOURNAL_COMMITTED) || resume {
+            let rollback = !resume || !self.is_set(JOURNAL_COMMITTED);
             #[cfg(any(feature = "use_pspd", feature = "use_vspd"))] {
                 if rollback {
                     self.spd.rollback();
@@ -463,6 +471,9 @@ impl<A: MemPool> Journal<A> {
                 let z = A::pre_dealloc(page.as_mut_ptr() as *mut u8, std::mem::size_of::<Page<A>>());
                 A::log64(A::off_unchecked(self.pages.off_ref()), nxt.off(), z);
                 A::perform(z);
+
+                #[cfg(feature = "check_allocator_cyclic_links")]
+                debug_assert!(A::verify());
             }
         }
         // if let Ok(prev) = A::deref_mut::<Self>(self.prev_off) {
@@ -506,7 +517,7 @@ impl<A: MemPool> Journal<A> {
     ///
     /// [`Chaperon::transaction`]: ../chaperon/struct.Chaperon.html#method.transaction
     ///
-    pub fn fast_forward(&self) -> bool {
+    pub fn resume(&self) -> bool {
         if !self.is_set(JOURNAL_COMMITTED) {
             false
         } else {
@@ -514,11 +525,7 @@ impl<A: MemPool> Journal<A> {
                 let s = String::from_utf8(self.chaperon.to_vec()).unwrap();
                 let c = unsafe { Chaperon::load(&s)
                     .expect(&format!("Missing chaperon file `{}`", s)) };
-                if c.completed() {
-                    true
-                } else {
-                    false
-                }
+                c.completed()
             } else {
                 true
             }
